/*
 *	Sherlock Library -- Structure of Search Engine Databases
 *
 *	(c) 1997 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

/* Structure of each database item: */

struct sdbitem {
  ulg oid;				/* Object ID */
  byte tag;				/* Section tag */
  byte flags;				/* High 3 bits: word type */
					/* Low 5 bits: word quality */
} __attribute__((packed));

#define MAX_TAG 256
#define ANY_TAG 0xff

#define WT_SHIFT 5
#define WQ_MASK 0x1f
#define WT_MASK 0xe0

#define WORDTYPE(flags) (((flags) & WT_MASK) >> WT_SHIFT)

/* Known word types (update search/daemon/lex.c when changing!) */

#define WT_WORD 0			/* Normal text word */
#define WT_REF 1			/* URL Reference */
#define WT_REF_APPLET 2			/* Applet Reference */
#define WT_REF_IMG 3			/* Image Reference */
#define WT_REF_SELF 4			/* Self-URL Reference */
#define WT_REF_FORM 5			/* Form Reference */
#define WT_PSEUDO_WORD 7		/* Raw string in non-word list */

/* Maximal length of word */

#define MAX_WORD_LEN 1024		/* URL should fit here */

/* Length of file page */

#define FILE_PAGE_SIZE 4096

/* Signature record in the SBTree */

struct sig {
  byte sig[8];
  ulg ptr;
};

/* Parameters of the SBTree */

#define TREE_PAGE_SIZE 1024
#define TREE_PAGE_DATA (TREE_PAGE_SIZE - sizeof(ulg))
#define KEYS_PER_PAGE (TREE_PAGE_DATA / sizeof(struct sig))
#define PAD_PER_PAGE (TREE_PAGE_DATA - sizeof(struct sig)*KEYS_PER_PAGE)

struct tree_page {
  struct sig keys[KEYS_PER_PAGE];	/* The keys */
  ulg aux;				/* Auxilliary data: for root page: total # of keys in the tree */
  byte pad[PAD_PER_PAGE];		/* Padding to end of the page */
};

/* Non-word hashing */

#define HASH_MAGIC 23
