/*
 *	Sherlock Search Engine -- Lexical Analysis of Queries
 *
 *	(c) 1997 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "sherlock.h"

#include <lib/string.h>

#include "parse.tab.h"

static byte *pos;

static byte *ref_keywords[] = {
  "REF", "REFURL", "REFAPPLET", "REFIMG", "REFSELF", "REFFORM", NULL
};

void
lex_init(byte *buf)
{
  pos = buf;
}

int
yylex(void)
{
  while (Cspace(*pos))
    pos++;
  if (!*pos)
    return EOLN;
  if (Cdigit(*pos))
    {
      uns i = 0;
      while (Cdigit(*pos))
	{
	  i = 10*i + *pos++ - '0';
	  if (i > 1000000)
	    err("Number too large");
	}
      yylval.i = i;
      return NUM;
    }
  if (Calpha(*pos))
    {
      byte *start = pos;
      byte x;
      uns k;

      while (Calpha(*pos))
	{
	  *pos = Cupcase(*pos);
	  pos++;
	}
      x = *pos;
      *pos = 0;
      if (!strcmp(start, "SECTION"))
	k = SECTION;
      else if (!strcmp(start, "FROM"))
	k = FROM;
      else if (!strcmp(start, "TO"))
	k = TO;
      else if (!strcmp(start, "AND"))
	k = AND;
      else if (!strcmp(start, "OR"))
	k = OR;
      else if (!strcmp(start, "NOT"))
	k = NOT;
      else if (!strcmp(start, "ANY"))
	k = ANY;
      else if (!strcmp(start, "CONTROL"))
	k = CONTROL;
#ifdef DEBUG
      else if (!strcmp(start, "XYZZY"))
	k = XYZZY;
#endif
      else
	{
	  byte **z = ref_keywords;
	  while (*z && strcmp(start, *z))
	    z++;
	  if (!*z)
	    err("Unknown keyword");
	  yylval.i = z - ref_keywords;
	  k = REF;
	}
      *pos = x;
      return k;
    }
  switch (*pos++)
    {
    case '"':
      {
	byte *z = pos;
	while (*pos && *pos != '"')
	  pos++;
	if (!*pos)
	  err("Misquoted string");
	*pos++ = 0;
	yylval.s = z;
	return STRING;
      }
    case '&':
      return AND;
    case '|':
      return OR;
    case '!':
      return NOT;
    case ',':
    case '(':
    case ')':
      return pos[-1];
    default:
      err("Unknown character");
    }
}
