/*
 *	Sherlock Search Engine -- Query Parser
 *
 *	(c) 1997, 1998 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>

#include "sherlock.h"

#include <lib/pools.h>
#include <lib/searchdb.h>

#define yyerror(x) err(x)

static byte *error;

static struct expr *new_node(uns t)
{
  struct expr *n = fast_alloc(current_query->pool, sizeof(struct expr));
  n->type = t;
  return n;
}

%}

%union {
  int i;
  struct query *q;
  struct expr *x;
  byte *s;
}

/* Tokens */

%token EOLN SECTION FROM TO ANY XYZZY CONTROL
%token <i> REF NUM
%token <s> STRING

%left OR
%left AND
%left NOT

%type <x> expr
%type <i> refmask quality rlist
%type <q> general

%%

main:
   general expr EOLN {
      $1->expr = $2;
      if (!$1->sections_set)
	memset($1->sections, 0x0ff, sizeof($1->sections));
      return 0;
   }
 | general CONTROL STRING {
      $1->expr = NULL;
      $1->cmd = $3;
      return 0;
   }
 ;

general:
   /* empty */ {
      $$ = current_query;
      $$->to = ~0;
   }
 | general XYZZY NUM {
      $1->debug = $3;
      $$ = $1;
   }
 | general SECTION seclist {
      $1->sections_set = 1;
      $$ = $1;
   }
 | general FROM NUM {
      $1->from = $3;
      $$ = $1;
   }
 | general TO NUM {
      $1->to = $3;
      $$ = $1;
   }
 ;

seclist:
   secno
 | seclist ',' secno
 ;

secno:
   NUM {
      current_query->sections[$1 >> 3] |= (1 << ($1 & 7));
   }
 ;

expr:
   '(' expr ')' {
     $$ = $2;
   }
 | expr AND expr {
     $$ = new_node(EX_AND);
     $$->u.op.l = $1;
     $$->u.op.r = $3;
   }
 | expr OR expr {
     $$ = new_node(EX_OR);
     $$->u.op.l = $1;
     $$->u.op.r = $3;
   }
 | NOT expr {
     $$ = new_node(EX_NOT);
     $$->u.op.l = $2;
     $$->u.op.r = NULL;
   }
 | refmask quality STRING {
     $$ = new_node(EX_MATCH);
     $$->u.match.quality = $2;
     $$->u.match.classmap = $1 ? $1 : (1 << WT_WORD);
     $$->u.match.is_nonword = !! $1;
     if (! $$->u.match.is_nonword && !have_word_list)
       {
	 $$->u.match.classmap = (1 << WT_PSEUDO_WORD);
	 $$->u.match.is_nonword = 1;
	 $$->u.match.word = $3;
       }
     else
       $$->u.match.word = ($$->u.match.is_nonword ? normalize_nonword : normalize_word) (current_query->pool, $3);
   }
| ANY {
     $$ = new_node(EX_ANY);
   }
 ;

refmask:
   /* empty */ {
     $$ = 0;
   }
 | rlist
 ;

rlist:
   REF {
     $$ = $1 ? (1 << $1) : ~(1 << WT_WORD);
   }
 | rlist ',' REF {
     $$ = $1 | ($3 ? (1 << $3) : ~(1 << WT_WORD));
   }
 ;

quality:
   /* empty */ {
     $$ = default_word_weight;
   }
 | NUM {
     if ($1 < 0 || $1 > 1000)
       err("Quality out of range");
     $$ = $1;
   }
 ;

%%

static jmp_buf err_jmp;

void
err(byte *x)
{
  error = x;
  longjmp(err_jmp, 1);
}

byte *
parse_query(byte *b)
{
  error = NULL;
  lex_init(b);
  if (!setjmp(err_jmp))
    yyparse();
  return error;
}
